/*

	f[^K[

	Copyrights 2010,  Yoshi Hiroshi.

 */
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "dataLogger.h"

void sequenceInit();
void sequence(int);
char getChargeMode();
int getChargeTime();
int getPrevChargeTime();
int getCycleCount();

/*
 *      ADC 
 */
volatile unsigned char channelIndex;
volatile char nextChannel;
volatile int  dataCount;
volatile int lastVal[MAX_ADC_PORT];
volatile int minVal[MAX_ADC_PORT];
volatile int maxVal[MAX_ADC_PORT];
volatile long adcVal[MAX_ADC_PORT];
volatile long resultVal[MAX_ADC_PORT];
volatile char resultReady;
volatile long lastVolt;
volatile long minVolt;
volatile long maxVolt;
volatile long lastCurr;
volatile long minCurr;
volatile long maxCurr;

/*
 *      Timer/Counter
 */
volatile char timerInterrupt;

/*
 *      Console IO
 */
#define TX_BUFSIZE 400
#define RX_BUFSIZE 20
int	uartPutChar(char c, FILE *stream);
int	uartGetChar(FILE *stream);
void uartCheckAndSend();
int uartCheckAndRecv();

char cmnd[8];

FILE uart = FDEV_SETUP_STREAM(uartPutChar, uartGetChar, _FDEV_SETUP_RW);

/*
 *
 */
ISR(ADC_vect)
{
    int curVal = ADC;
    if(nextChannel < MAX_ADC_PORT) {
        ADMUX = _BV(REFS1) | _BV(REFS0) | nextChannel;    // Vref = 1.1V
        ADCSRA |= _BV(ADSC);
        nextChannel++;
    }
    adcVal[channelIndex] += curVal;
    lastVal[channelIndex] = curVal;
    if(dataCount == 0) {
        minVal[channelIndex] = curVal;
        maxVal[channelIndex] = curVal;
    } else {
        if(minVal[channelIndex] > curVal) minVal[channelIndex] = curVal;
        if(maxVal[channelIndex] < curVal) maxVal[channelIndex] = curVal;
    }
    channelIndex++;
    if(channelIndex >= MAX_ADC_PORT)
    {
        channelIndex = 0;
        dataCount++;
        if(dataCount >= MAX_DATA_COUNT)
        {
            dataCount = 0;
            resultVal[0] = adcVal[0] * 11000L / 1024L;
            resultVal[1] = adcVal[1] * 11000L / 1024L;
            if(ADC_CONV_CAP_VOLT) {
                resultVal[2] = adcVal[2] * (25000L/8) / (1024/8);   //tXP[␳i킹j
            } else {
                resultVal[2] = adcVal[2] * 11000L / 1024L;
            }
            if(ADC_CONV_CAP_CURR) {
                resultVal[3] = (adcVal[3] - ADC_BIPOLAR_ZERO) * 1090L / 512L / 2; // 2mV/1A
            } else {
                resultVal[3] = adcVal[3] * 11000L / 1024L;
            }
            adcVal[0] = 0;
            adcVal[1] = 0;
            adcVal[2] = 0;
            adcVal[3] = 0;
            lastVolt = lastVal[2];
            minVolt  = minVal[2];
            maxVolt  = maxVal[2];
            lastCurr = lastVal[3];
            minCurr  = minVal[3];
            maxCurr  = maxVal[3];
            resultReady = 1;
        }
    }
}

/*
 *
 */
ISR(TIMER1_COMPA_vect)
{
    ADMUX = _BV(REFS1) | _BV(REFS0);    // Vref = 1.1V
    ADCSRA |= _BV(ADSC);
    nextChannel = 1;
    channelIndex = 0;
    timerInterrupt = 1;
}

/*
 *
 */
void sendVal(long val, char* prefix)
{
    int sign = 1;
    printf("%s", prefix);
    if(val < 0) {
        sign = -1;
//        printf("-");
        val = -val;
    }
    if(val >= 1000000) {
        printf("%2d", (int)(val/1000000) * sign);
        printf("%03d", (int)((val/1000) % 1000));
    } else {
        printf("%3d", (int)(val/1000) * sign);
    }
    printf(".%03d", (int)(val%1000));
}

/*
 *
 */
void sendData()
{
    static int count1 = 0;
    static int count2 = 0;
    count1++;
    if(count1 >= 600) {
        count1 = 0;
        count2++;
        if(count2 >= (1440 * 21)) {
            count2 = 0;
        }
    }
    char m = getChargeMode();
    if(m != 0) {
        printf("CYCLE:%d.%d ", (getCycleCount()+1)/2, ((getCycleCount()+1)&0x01)*5);
    } else {
        printf("CYCLE:%d   ", (getCycleCount()+1)/2);
    }
    printf("%5d", count2);
    printf(".%03d", count1);

    printf(",Last    Min     Max ,VOLT ");
    if(ADC_CONV_CAP_VOLT) {
        sendVal(lastVolt * (25000L/8L) / (10240L/8L), " ");
        sendVal(minVolt * (25000L/8L) / (10240L/8L), " ");
        sendVal(maxVolt * (25000L/8L) / (10240L/8L), " ");
    } else {
        sendVal(lastVolt * 1000L / 1024L, " ");
        sendVal(minVolt * 1000L / 1024L, " ");
        sendVal(maxVolt * 1000L / 1024L, " ");
    }
    printf(",raw  %7d", (int)lastVolt);
    printf(" %7d", (int)minVolt);
    printf(" %7d", (int)maxVolt);

    printf(",CURR ");
    if(ADC_CONV_CAP_CURR) {
        sendVal((lastCurr - 512L) * 109000L /(512L * 2L), " ");
        sendVal((minCurr - 512L) * 109000L /(512L * 2L), " ");
        sendVal((maxCurr - 512L) * 109000L /(512L * 2L), " ");
    } else {
        sendVal(lastCurr * 1100L / 1024L, " ");
        sendVal(minCurr * 1100L / 1024L, " ");
        sendVal(maxCurr * 1100L / 1024L, " ");
    }
    printf(",raw  %7d", (int)lastCurr);
    printf(" %7d", (int)minCurr);
    printf(" %7d", (int)maxCurr);

//    sendVal(resultVal[0]);
//    sendVal(resultVal[1]);
    sendVal(resultVal[2], ",");
    sendVal(resultVal[3], ",");

    long currEnergy = (resultVal[2]/10) * (resultVal[3]/10) / 100000L;
    sendVal(currEnergy, ",");

    printf("\n");
}

/*
 *
 */
void init()
{   // Clock Freq. = 20MHz
    // q[YCKDIV8̃`FbNt܂܂̎A2sLɂ
    // CLKPR = _BV(CLKPCE);
    // CLKPR = 0;

    DDRB = 0x03;
    DDRD = 0x0E;
    PORTD = 0xF0;
// ADC
    ADMUX = _BV(REFS1) | _BV(REFS0);
    ADCSRA = _BV(ADEN) | _BV(ADIE) | _BV(ADPS2) | _BV(ADPS1) | _BV(ADPS0);
// COUNTER
    OCR1A = (unsigned int)(1000000L/50 - 1);        // 1000uS
    TCCR1B = _BV(WGM12) | _BV(CS10);                // 50nS
    TIMSK1 |= _BV(OCIE1A);
// USART
    UCSR0C = _BV(UCSZ01) | _BV(UCSZ00);
    UCSR0B = _BV(RXEN0) | _BV(TXEN0);
    UCSR0A = _BV(U2X0);
    UBRR0 = 42;                             // 57600
//    UBRR0 = 64;                             // 38400
// stdio
    stdout = stdin = &uart;
    stderr = &uart;
// working strage
    resultReady = 0;
    dataCount = 0;
    nextChannel = 1;
    channelIndex = 0;
    timerInterrupt = 0;
}

/*====================================================================
 *
 *                            main routine
 *
 ====================================================================*/
int main()
{
    init();
    sequenceInit();
    ADCSRA |= _BV(ADSC);
    sei();

    printf("DATA LOGGER STARTED.\n");

    while(1)
    {
        if(resultReady)
        {
            resultReady = 0;
            sendData();
        }
        if(timerInterrupt)
        {
            timerInterrupt = 0;
            sequence(lastVal[2]);
        }
        uartCheckAndSend();
        if(uartCheckAndRecv())
        {
        }
    }
}

//*********************************************************************
//
//
char txBuff[TX_BUFSIZE];
char * txBufPtrIn = txBuff;
char * txBufPtrOut = txBuff;
char rxBuff[RX_BUFSIZE];
char * rxBufPtrIn = rxBuff;
char * rxBufPtrOut = rxBuff;

/**********************************************************************
 *
 ---------------------------------------------------------------------*/
void uartPutTxBuff(char c)
{
    char * nextPtr = txBufPtrIn;
    nextPtr++;
    if(nextPtr >= &txBuff[TX_BUFSIZE])
    {
        nextPtr = txBuff;
    }
    if(nextPtr != txBufPtrOut)
    {
        *txBufPtrIn = c;
        txBufPtrIn = nextPtr;
    }
}

/**********************************************************************
 *
 ---------------------------------------------------------------------*/
int uartGetTxBuff()
{
    if(txBufPtrIn == txBufPtrOut) return 0;
    char c = *txBufPtrOut++;
    if(txBufPtrOut >= &txBuff[TX_BUFSIZE])
    {
        txBufPtrOut = txBuff;
    }
    return c;
}

/**********************************************************************
 *
 ---------------------------------------------------------------------*/
void uartCheckAndSend()
{
    char c;
    if(txBufPtrIn != txBufPtrOut)
    {
        if(UCSR0A & _BV(UDRE0))
        {
            if((c = uartGetTxBuff()) != 0)
            {
                UDR0 = c;
            }
        }
    } else {
    }
}

/**********************************************************************
 *
 ---------------------------------------------------------------------*/
void uartPutRxBuff(char c)
{
    char * nextPtr = rxBufPtrIn;
    nextPtr++;
    if(nextPtr >= &rxBuff[RX_BUFSIZE])
    {
        nextPtr = rxBuff;
    }
    if(nextPtr != rxBufPtrOut)
    {
        *rxBufPtrIn = c;
        rxBufPtrIn = nextPtr;
    }
}

/**********************************************************************
 *
 ---------------------------------------------------------------------*/
int uartGetRxBuff()
{
    if(rxBufPtrIn == rxBufPtrOut) return 0;
    char c = *rxBufPtrOut++;
    if(rxBufPtrOut >= &rxBuff[RX_BUFSIZE])
    {
        rxBufPtrOut = rxBuff;
    }
    return c;
}

/**********************************************************************
 *
 ---------------------------------------------------------------------*/
int uartCheckAndRecv()
{
    if(UCSR0A & _BV(RXC0))
    {
        uartPutRxBuff(UDR0);
    }
	if (UCSR0A & _BV(FE0))
	  return _FDEV_EOF;
	if (UCSR0A & _BV(DOR0))
	  return _FDEV_ERR;
    return !(rxBufPtrIn == rxBufPtrOut);
}

/**********************************************************************
 *
 *          Standart I/O Interface                  uartPutChar
 *
 *          Xg[֕1o͂
 *
 **********************************************************************/
int uartPutChar(char c, FILE *stream)
{
    if (c == '\n')
        uartPutTxBuff('\r');
    uartPutTxBuff(c);
    return 0;
}

/**********************************************************************
 *
 *          Standart I/O Interface                  uartGetChar
 *
 *          Xg[當1͂
 *
 **********************************************************************/
int uartGetChar(FILE *stream)
{
  return uartGetRxBuff();
}

