/*

	f[^K[

	Copyrights 2010,  Yoshi Hiroshi.

 */

#include <avr/io.h>
#include "dataLogger.h"

volatile char mode;
volatile char modeView;

volatile char voltOver;
volatile char voltUnder;
volatile long timer;
volatile long timerGap;
volatile long chargeTime;
volatile long prevChargeTime;
volatile char cycleStart;
volatile int cycleCount;

/*
 *      Initialize
 */
void sequenceInit()
{
    CHARGE_DISABLE;
    DISCHARGE_DISABLE;
    mode = MODE_STOP;
    voltOver = 0;
    voltUnder = 0;
    timer = 0;
    timerGap = 0;
    chargeTime = 0;
    prevChargeTime = 0;
    cycleStart = 0;
    cycleCount = 0;
}

/*
 *      Check for charge mode
 */
char getChargeMode()
{
    char result;
    if(modeView == MODE_CHARGE) {
        result = MODE_CHARGE;
    } else if(modeView == MODE_DISCHARGE) {
        result = MODE_DISCHARGE;
    } else {
        result = 0;
    }
    if(mode != modeView) modeView = mode;
    return result;
}


/*
 *      Get ChargeTime
 */
int getChargeTime()
{
    return (int)(chargeTime/1000);
}

/*
 *      Get PrevChargeTime
 */
int getPrevChargeTime()
{
    return (int)(prevChargeTime/1000);
}

/*
 *      Get CycleCount
 */
int getCycleCount()
{
    return cycleCount;
}


/*
 *      Sequence Control
 *
 *      process every 1mS
 */
void sequence(int volt)
{
    chargeTime++;
    long realVolt = volt * (2500000L/8) / (1024/8);
    if(realVolt > 2100000) {    // 2.10V𒴂A[d~
        voltUnder = 0;
        if(voltOver < 10) voltOver++;
        if(voltOver >= 3) {
            CHARGE_DISABLE;
            LED_CLEAR_RED;
            prevChargeTime = chargeTime;
            chargeTime = 0;
            if(mode == MODE_CHARGE) {
                mode = MODE_IDLE1;
                timerGap = TIME_CYCLE_GAP;
            }
        }
    }
    if(realVolt < 700000) {     // 0.7VȉȂAd~
        voltOver = 0;
        voltUnder++;
        if(voltUnder < 10) voltUnder++;
        if(voltUnder >= 3) {
            DISCHARGE_DISABLE;
            LED_CLEAR_YELLOW;
            prevChargeTime = chargeTime;
            chargeTime = 0;
            if(mode == MODE_DISCHARGE) {
                mode = MODE_IDLE2;
                timerGap = TIME_CYCLE_GAP;
            }
        }
    }
    if(timer < 10000000) timer++;
    if(timerGap > 0) {
        timerGap--;
    } else {
        if((mode == MODE_IDLE1) && (timer >= TIME_CYCLE_CHARGE)) {
            mode = MODE_DISCHARGE;
            modeView = MODE_DISCHARGE;
            DISCHARGE_ENABLE;
            LED_SET_YELLOW;
            timer = 0;
            chargeTime = 0;
            if(cycleStart != 0) {
                cycleCount++;
                cycleStart = 0;
            }
        }
        if((mode == MODE_IDLE2) && (timer >= TIME_CYCLE_DISCHARGE)) {
            mode = MODE_CHARGE;
            modeView = MODE_CHARGE;
            CHARGE_ENABLE;
            LED_SET_RED;
            timer = 0;
            chargeTime = 0;
            if(cycleStart != 1) {
                cycleCount |= 1;
                cycleStart = 1;
            }
        }
    }
    if((mode == MODE_STOP) && (timer >= TIME_INITIAL_WAIT)) {
        mode = MODE_CHARGE;
        modeView = MODE_CHARGE;
        CHARGE_ENABLE;
        LED_SET_RED;
        timer = 0;
        chargeTime = 0;
        if(cycleStart != 1) {
            cycleCount |= 1;
            cycleStart = 1;
        }
    }
}
