/*

	pd[d

	Copyrights 2010,  Yoshi Hiroshi.

 */

#include <avr/io.h>
#define F_CPU 16000000UL
#include <util/delay.h>

#define ADC_GET_VOLT            (_BV(REFS1) | 0x01)
#define ADC_GET_CURR            (_BV(REFS1) | 0x00)
#define VOLTAGE_LIMIT           ((1200L * 1024L * ADC_COUNT_MAX * 39L) / (2560L * 200L))   // 2.40V
#define VOLTAGE_LIMIT_50V_R2    ((5283L * 1024L * ADC_COUNT_MAX * 39L) / (2560L * 200L))   // 5.00V
#define VOLTAGE_LIMIT_25V_R2    ((2560L * 1024L * ADC_COUNT_MAX * 39L) / (2560L * 200L))   // 2.50V
#define VOLTAGE_LIMIT_50V_R1    ((5210L * 1024L * ADC_COUNT_MAX * 39L) / (1100L * 200L))   // 5.00V
#define VOLTAGE_LIMIT_25V_R1    ((2570L * 1024L * ADC_COUNT_MAX * 39L) / (1100L * 200L))   // 2.50V
#define CURRENT_LIMIT_30A       (((30L * (40L+ 1L) * 1024L) * ADC_COUNT_MAX) / 2560L)   // 2mVx20/1A
#define CURRENT_LIMIT_25A       (((25L * (40L+ 1L) * 1024L) * ADC_COUNT_MAX) / 2560L)   // 2mVx20/1A
#define CURRENT_LIMIT_20A       (((20L * (40L+ 1L) * 1024L) * ADC_COUNT_MAX) / 2560L)   // 2mVx20/1A
#define CURRENT_LIMIT_15A       (((15L * (40L- 0L) * 1024L) * ADC_COUNT_MAX) / 2560L)   // 2mVx20/1A
#define CURRENT_LIMIT_10A       (((10L * (40L- 1L) * 1024L) * ADC_COUNT_MAX) / 1100L)   // 2mVx20/1A
#define CURRENT_LIMIT_8A        ((( 8L * (40L- 2L) * 1024L) * ADC_COUNT_MAX) / 1100L)   // 2mVx20/1A
#define CURRENT_LIMIT_5A        ((( 5L * (40L- 4L) * 1024L) * ADC_COUNT_MAX) / 1100L)   // 2mVx20/1A
#define CURRENT_LIMIT_3A        ((( 3L * (40L-11L) * 1024L) * ADC_COUNT_MAX) / 1100L)   // 2mVx20/1A
//#define CURRENT_LIMIT_1A        ((( 1L * (40L-28L) * 1024L) * ADC_COUNT_MAX) / 1100L)   // 2mVx20/1A
#define ADC_COUNT_MAX               8

#define RUNNING_MODE_STOP           0x10
#define RUNNING_MODE_STARTINGUP     0x20
#define RUNNING_MODE_RUNNING        0x30

volatile char runningMode;
volatile char prevSwitch;
volatile char currSwitch;
volatile unsigned char adcState;
volatile unsigned char adcCount;
volatile int startingDelayCount;
volatile long voltageLimit;
volatile long voltage;
volatile long currentLimit;
volatile long current;
volatile int adcValue;
volatile int pwmValue;
volatile long currError;
volatile long prevError;
volatile long currControl;
volatile long prevControl;

#define PWM_MAX_VALUE               (639)

void init()
{
    PORTA = 0xF0;
    PORTB = 0xFF;
    DDRB = 0x30;
    runningMode = RUNNING_MODE_STOP;
    prevSwitch = ~(PINA >> 4);

    _delay_ms(500);
    ADCSRA = _BV(ADEN) | _BV(ADPS2);
    ADCSRB = 0;
    ADMUX = ADC_GET_VOLT;
    adcState = ADC_GET_VOLT;
    ADCSRA |= _BV(ADSC);
    adcCount = 0;
    voltage = 0;
    current = 0;

//    PLLCSR |= _BV(PLLE);
    PLLCSR |= _BV(PCKE);
    DT1 = 0x00;
    TCCR1C = 0;
    PORTB |= 0x30;
    TC1H = (PWM_MAX_VALUE >> 8);
    OCR1C = PWM_MAX_VALUE & 0xFF;
    TC1H = 0;
    OCR1D = 0;
    OCR1A = 0;
    TC1H = 2;
    OCR1B = 0xFF;
    TCCR1B = 0x80 | _BV(CS10);
}

void setNewControl(char runMode, int volt, int curr)
{
    if(runMode != RUNNING_MODE_STOP) {
        if(voltageLimit > volt) {
        //if(curr > currentLimit) {
            currError = (currentLimit - curr) >> 5;
            if(runMode == RUNNING_MODE_STARTINGUP)
            {
                if(currError <= 0) {
                    runMode = RUNNING_MODE_RUNNING;
                    TCCR1C = _BV(COM1D0) | _BV(PWM1D);  // 
                } else {
                    if(currError > 4)
                        currError = 4;
                }
            }
        } else {
            currError = (voltageLimit - volt) >> 2;
            if(runMode == RUNNING_MODE_STARTINGUP) {
                if(currError <= 0) {
                    runMode = RUNNING_MODE_RUNNING;
                    TCCR1C = _BV(COM1D0) | _BV(PWM1D);  // 
                } else {
                    if(currError > 4)
                        currError = 4;
                }
            }
        }
        currControl = (currError - prevError / 2) + prevControl;
        if(currControl < 0) currControl = 0;
        if(currControl > (PWM_MAX_VALUE * ADC_COUNT_MAX)) 
            currControl = PWM_MAX_VALUE * ADC_COUNT_MAX;

        prevError = currError;
        prevControl = currControl;
        currControl = currControl / ADC_COUNT_MAX;
        TC1H = (currControl >> 8);
        OCR1D = (currControl & 0xFF);
    }
}

void getAdcData()
{
    adcValue = ADC;
    if(adcState == ADC_GET_VOLT) {
        voltage += adcValue;
        adcState = ADC_GET_CURR;
        ADMUX = ADC_GET_CURR;
        ADCSRA |= _BV(ADSC);
    } else {
        current += adcValue;
        adcState = ADC_GET_VOLT;
        ADMUX = ADC_GET_VOLT;
        ADCSRA |= _BV(ADSC);
    }
    adcCount++;
    if(adcCount >= (ADC_COUNT_MAX * 2)) {
        adcCount = 0;
        if(runningMode != RUNNING_MODE_STOP) {
            setNewControl(runningMode, voltage, current);
        }
        voltage = 0;
        current = 0;
    }
}

void getRange()
{
    switch(currSwitch & 0x07) {     // current mode
    case 0:
        ADCSRB |= _BV(REFS2);
        currentLimit = CURRENT_LIMIT_30A;
        break;
    case 1:
        ADCSRB |= _BV(REFS2);
        currentLimit = CURRENT_LIMIT_25A;
        break;
    case 2:
        ADCSRB |= _BV(REFS2);
        currentLimit = CURRENT_LIMIT_20A;
        break;
    case 3:
        ADCSRB |= _BV(REFS2);
        currentLimit = CURRENT_LIMIT_15A;
        break;
    case 4:
        ADCSRB &= ~_BV(REFS2);
        currentLimit = CURRENT_LIMIT_10A;
         break;
    case 5:
        ADCSRB &= ~_BV(REFS2);
        currentLimit = CURRENT_LIMIT_5A;
         break;
    case 6:
        ADCSRB &= ~_BV(REFS2);
        currentLimit = CURRENT_LIMIT_3A;
         break;
    case 7:
        ADCSRB &= ~_BV(REFS2);
        currentLimit = CURRENT_LIMIT_8A;
         break;
    }

    switch(currSwitch & 0x08) {     // voltage mode
    case 0x08:
        if(ADCSRB &_BV(REFS2))
            voltageLimit = VOLTAGE_LIMIT_25V_R2;
        else
            voltageLimit = VOLTAGE_LIMIT_25V_R1;
        break;
    case 0x00:
        if(ADCSRB &_BV(REFS2))
            voltageLimit = VOLTAGE_LIMIT_50V_R2;
        else
            voltageLimit = VOLTAGE_LIMIT_50V_R1;
        break;
    }
}

int main()
{
    init();
    getRange();
    _delay_ms(500);

    while(1)
    {
        if(bit_is_set(ADCSRA, ADIF)) {
            ADCSRA |= _BV(ADIF);
            getAdcData();
        }
        if(bit_is_clear(PINB, 6)) {
            if(runningMode == RUNNING_MODE_STOP) {
                runningMode = RUNNING_MODE_STARTINGUP;
                prevControl = 0;
                TC1H = 0;
                OCR1D = 0;
                TCCR1C = _BV(COM1D1) | _BV(PWM1D);
            }
        } else {
            if(runningMode != RUNNING_MODE_STOP) {
                runningMode = RUNNING_MODE_STOP;
                TCCR1C = 0;         // stop pwm
                PORTB |= 0x30;
            }
        }
        currSwitch = PINA >> 4;
        if(prevSwitch != currSwitch)
        {
            getRange();
            prevSwitch = currSwitch;
        }
    }
}
