/*

	GlM[E[^

	dCdwLpV^̓d~dGlM[߂

	Copyrights 2010,  Yoshi Hiroshi.

 */


/*
    FUSE Settings:
        EXTENDED	0xFF
        HIGH		0xD6
        LOW		    0x62
            SELFPRGEN	NO-CHECK
            RSTDISBL	NO-CHECK
            DWEN		NO-CHECK
            SPIEN		CHECK
            WDRON		NO-CHECK
            EESAVE		CHECK
            BODLEVEL	Brown-out detection at VCC=1.8V
            CKDIB8		CHECK
            CHOUT		NO-CHECK
            SUT_CKSEL	Int. RC Osc. 8 MHz,
            		     Start-up time PWRDWN/RESET: 6 CK/14 CK + 64 ms
*/

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/sleep.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include "i2cTinyLcd.h"
#include "switch.h"
#include "setting.h"
#include "eeProm.h"

#define GET_CAP_VOLT(val)           ((long)val * (1100L * 248L) / (1024L * 101L))

#define ICON_BLINK_SPEED            4
volatile char iconBlinkCounter = 0;

#define DATA_AVERAGE_COUNT          8
volatile char dataCount;
volatile long totalVolt;
volatile long totalTemp;
volatile unsigned char sleepMode;
volatile unsigned char dispMode;

char msg_title1[] PROGMEM = { "EDL ߼      " };
char msg_title2[] PROGMEM = { "  ٷ Ұ" };

void wdtEnable()
{
    MCUSR &= ~_BV(WDRF);
    WDTCR |= _BV(WDCE) | _BV(WDE);
    WDTCR = _BV(WDIE) | _BV(WDP0);     // 32mS;
}

ISR(WDT_vect)
{
}

void initApplication()
{
    PORTB = 0x18;
    sleepMode = SLEEP_MODE_PWR_DOWN;
    switchInit();
    i2cInit();
    i2cLcdInit();
    i2cLcdSetContrast(GET_CONTRAST());
    printf_P(msg_title1);
    i2cLcdPosition(1, 0);
    printf_P(msg_title2);
    _delay_ms(2000);
}

void adcEnable()
{
    ADCSRA = _BV(ADEN) | _BV(ADPS2) | _BV(ADPS1) | _BV(ADPS0);
    ADMUX = _BV(REFS1);
}

inline void adcDisable()
{
    ADMUX = 0;
    ADCSRA = 0;
}

int getAdcVal(unsigned char chn)
{
    ADMUX = _BV(REFS1) | chn;
    ADCSRA |= _BV(ADSC);
    loop_until_bit_is_set(ADCSRA, ADIF);
    ADCSRA |= _BV(ADIF);
    return ADC;
}

void showBatteryIcon(long volt)
{
    char icon = 0x02;
    if(volt < BAT_METER_THRESHOLD(0)) {       // under 1.0V
        if(iconBlinkCounter & 0x01) icon &= ~0x02;
    }
    if(volt >= BAT_METER_THRESHOLD(0)) {      // Over 1.0V
        icon |= 0x10;
    }
    if(volt < BAT_METER_THRESHOLD(1)) {       // under 1.2V
        if(iconBlinkCounter & 0x01) icon &= ~0x10;
    }
    if(volt >= BAT_METER_THRESHOLD(2)) {
        icon |= 0x08;
    }
    if(volt < BAT_METER_THRESHOLD(3)) {       // Under 1.8V
        if(iconBlinkCounter & 0x02) icon &= ~0x08;
    }
    if(volt >= BAT_METER_THRESHOLD(4)) {       // Over 2.1V
        icon |= 0x04;
    }
    if(volt < BAT_METER_THRESHOLD(5)) {       // Under 2.3V
        if(iconBlinkCounter >= ICON_BLINK_SPEED) icon &= ~0x04;
    }
    if(volt > BAT_METER_THRESHOLD(6)) {       // Over 2.5V
        if(iconBlinkCounter & 0x01) {
            i2cLcdShowBattenIcon(0x10);
        } else {
            i2cLcdShowBattenIcon(0x00);
        }
    } else {
        i2cLcdShowBattenIcon(0x00);
    }
    i2cLcdShowBatteryIcon(icon);
    iconBlinkCounter++;
    if(iconBlinkCounter >= ICON_BLINK_SPEED*2) iconBlinkCounter = 0;
}

void measureCapacitance()
{
    adcEnable();
    totalVolt += getAdcVal(1);
    totalTemp += getAdcVal(0x0F);
    dataCount++;
    if(dataCount >= DATA_AVERAGE_COUNT) {
        dataCount = 0;
        long volt = totalVolt / DATA_AVERAGE_COUNT;
        volt = GET_CAP_VOLT(volt);
        volt = (volt + 5) / 10;        // 10mV̈ʂŎľܓ
        volt *= 10;
        long temp = (totalTemp - 300 * DATA_AVERAGE_COUNT) * 1400 / 
                    (DATA_AVERAGE_COUNT * 125) + 250;
        long cap = getCapacity(volt, temp);
        long energy = (((volt * volt) / 100L) * cap) / (2L * 1000L);

//        i2cLcdClear();
        if(dispMode == 0) {
            i2cLcdPosition(0, 0);
            printf("%2d.%02dV ", (int)(volt/1000), (int)((volt % 1000) / 10));
            printf("%4d.%1dC ", (int)(temp/10), (int)(temp % 10));
            i2cLcdPosition(1, 0);
            printf("%5dF ", (int)cap);
            printf("%5d.%1dWs", (int)(energy/10), (int)(energy % 10));
        }
        showBatteryIcon(volt);
        totalVolt = 0;
        totalTemp = 0;
    }
    adcDisable();
}


int main()
{
    CLKPR = _BV(CLKPCE);    // NbN̕ݒ苖
    CLKPR = 0x00;           // NbN̕Ȃ
    initApplication();      // 
    i2cLcdClear();          // NbZ[WNA
    wdtEnable();
    dispMode = 0;

    while(1)
    {
        set_sleep_mode(sleepMode);
        sleep_enable();
        sei();
        sleep_cpu();                // EHb`hbO̔҂
        sleep_disable();

        measureCapacitance();       // E\
        checkSwitches();            // XBb`͂`FbN
        dispMode = checkBottun(dispMode);   // {^s
    }
}
