/*

	GlM[E[^

	i2cTinyLcd.c

	IICڑLCDW[Rg[B

	Copyrights 2010,  Yoshi Hiroshi.

 */



#include "i2cTinyLcd.h"
#include <avr/eeprom.h>
#include <avr/pgmspace.h>

unsigned char lcdContrast;

#ifdef I2C_LCD_USE_STDIO_FUNCTION
FILE i2cLcd = FDEV_SETUP_STREAM(i2cLcdPutChar, i2cLcdGetChar, _FDEV_SETUP_RW);

int	i2cLcdPutChar(char c, FILE *stream)
{
    i2cLcdData(c);
    return 0;
}

int	i2cLcdGetChar(FILE *stream)
{
    return 0;
}

#endif

void i2cLcdPuts(char* str)
{
    while(*str != 0) i2cLcdData(*str++);
}

void i2cLcdPutsE(char* str)
{
	char ch;
	while((ch = eeprom_read_byte((unsigned char*)str)) != 0)
	{
		str++;
		i2cLcdData(ch);
	}
}

void i2cLcdPutsP(char* str)
{
	char ch;
	while((ch = pgm_read_byte((unsigned char*)str)) != 0)
	{
		str++;
		i2cLcdData(ch);
	}
}

void i2cLcdClear()
{
    i2cLcdCmd(LCD_COMMAND_CLEAR);
    _delay_ms(2);
}

void i2cLcdPosition(char row, char col)
{
    i2cLcdCmd(LCD_COMMAND_SET_ADDRESS | ((row & 0x01) << 6) | (col & 0x1F));
}

int  i2cLcdGetContrast()
{
    return lcdContrast;
}

void i2cLcdSetContrast(int con)
{
    lcdContrast = con & 0x3F;
	i2cLcdCmd(LCD_COMMAND_INSTRUCTION_TBL_SEL); // function set
	i2cLcdCmd(LCD_COMMAND_CONTRAST_L | (lcdContrast & 0xF)); // contrast Low
	i2cLcdCmd(LCD_COMMAND_CONTRAST_H | ((lcdContrast >> 4) & 0x3)); // contast High/icon/power
}

void i2cLcdShowBatteryIcon(int icon)
{
	i2cLcdCmd(LCD_COMMAND_INSTRUCTION_TBL_SEL); // function set
	i2cLcdCmd(LCD_COMMAND_SET_ICON_ADDRESS | 0x0D); // contrast Low
    i2cLcdData(icon);
}

void i2cLcdShowBattenIcon(int icon)
{
	i2cLcdCmd(LCD_COMMAND_INSTRUCTION_TBL_SEL); // function set
	i2cLcdCmd(LCD_COMMAND_SET_ICON_ADDRESS | 0x0B); // contrast Low
    i2cLcdData(icon);
}

void i2cLcdCmd(char c)
{
    i2cWrite2(I2C_LCD_SLAVEADDRESS, I2C_LCD_CONTROL_CMND, c);
    _delay_us(50);
}

void i2cLcdData(char c)
{
    i2cWrite2(I2C_LCD_SLAVEADDRESS, I2C_LCD_CONTROL_DATA, c);
}

void i2cLcdInit()
{
    lcdContrast = I2C_LCD_DEFAULT_CONTRAST;
	// I2C LCD̏s܂B
	_delay_ms(40);
	i2cLcdCmd(LCD_COMMAND_FUNCTIONSET);         // function set
	i2cLcdCmd(LCD_COMMAND_INSTRUCTION_TBL_SEL); // g
	i2cLcdCmd(LCD_COMMAND_OSC_FREQ);            // IV[^̐ݒ
	i2cLcdCmd(LCD_COMMAND_CONTRAST_L | (lcdContrast & 0xF)); // RgXgݒi4rbgj
	i2cLcdCmd(LCD_COMMAND_CONTRAST_H | ((lcdContrast >> 4) & 0x3)); // RgXgʐݒ
	i2cLcdCmd(LCD_COMMAND_FOLLOWER);            // follower control
	_delay_ms(200);

	i2cLcdCmd(LCD_COMMAND_FUNCTIONSET);         // function set
	i2cLcdCmd(LCD_COMMAND_DISP_CONTROL | LCD_COMMAND_DISPCTRL_DISP_ON); // fBXvC\ON
	
	i2cLcdCmd(LCD_COMMAND_CLEAR);               // \NA
	_delay_ms(2);			 // Clear Display͒ǉEFCgKv

#ifdef I2C_LCD_USE_STDIO_FUNCTION	
// stdio
    stdout = stdin = &i2cLcd;
    stderr = &i2cLcd;
#endif
}

//------------------------------
void i2cInit()
{
	I2C_SCL_FREE;
	I2C_SDA_FREE;
}

//------------------------------
void i2cSendStartCondition()
{
	I2C_SCL_FREE;			// free SCL line
	while( I2C_SCL_CURR_VAL == 0 );
	_delay_us(DELAY_LONG);
	I2C_SDA_SET_LOW;		// Initiate SDA line
	_delay_us(DELAY_LONG);
	I2C_SCL_SET_LOW;		// Initiate SCL line
}

//------------------------------
void i2cSendStopCondition()
{
	I2C_SDA_SET_LOW;
	_delay_us(DELAY_SHORT);
	I2C_SCL_FREE;
	while( I2C_SCL_CURR_VAL == 0 );
	_delay_us(DELAY_SHORT);
	I2C_SDA_FREE;
	_delay_us(DELAY_LONG);
}

//------------------------------
unsigned char i2cWriteData(unsigned char val)
{
	unsigned char result;
	for(unsigned char count = 0; count < 8; count++)
    {
        if(val & 0x80) {
        	I2C_SDA_FREE;			// send 1
        } else {
        	I2C_SDA_SET_LOW;	    // send 0
        }
		_delay_us(DELAY_LONG);
		I2C_SCL_FREE;               // toggle SCL to high(free)
		while( I2C_SCL_CURR_VAL == 0 );
		_delay_us(DELAY_LONG);
		I2C_SCL_SET_LOW;            // toggle SCL to low
        val = val << 1;
	}
    I2C_SDA_FREE;
	_delay_us(DELAY_LONG);
    I2C_SCL_FREE;               // toggle SCL to high(free)
	_delay_us(DELAY_LONG);
    result = I2C_SDA_CURR_VAL;
	I2C_SCL_SET_LOW;            // toggle SCL to low
	return (result == 0);
}

//------------------------------
unsigned char i2cWrite2(unsigned char slaveAdrs, unsigned char control, unsigned char data)
{
// o Send start
	i2cSendStartCondition();
// o Send Slave address & read/write flga
	if(i2cWriteData(((slaveAdrs << 1) | I2C_REQUEST_WRITE)))
	{
// o Send Control
		if(i2cWriteData(control))
		{
// o Send Data
		    if(i2cWriteData(data))
            {
				//	return 0;
            }
		}
	}
// o Send Stop
	i2cSendStopCondition();
//
	return 0;
}
