/*

	񃂃j^

	AVR:ATtiny13A

	Copyrights 2010,  Yoshi Hiroshi.

 */

#include <avr/io.h>
#include <avr/sleep.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>

#define REF_VOLT                    1100L
#define LIMIT_MONITOR_VOLT          1800L
#define LIMIT_LOW_VOLT              2100L
#define LIMIT_FULL_VOLT             2300L
#define LIMIT_WARNING_VOLT          2450L
#define VOLT_CHANGE_DEADBAND        10

#define CONV_CAP_VOLT(v)            (((long)(v) * 265L * REF_VOLT) / (110L * 1024L))
#define CONV_LIMIT(l)               (((long)(l) * 110L * 1024L) / (REF_VOLT * 265L))

#define BYPASS_ON                   {setPwm(0);PORTB |= 0x02;}
#define BYPASS_OFF                  {setPwm(0);PORTB &= ~0x02;}
#define BYPASS_PWM(d)               {setPwm(d);}
#define CHARGE_END_ON               {if(capVolt > CONV_LIMIT(LIMIT_MONITOR_VOLT)) PORTB |= 0x01;}
#define CHARGE_END_OFF              {PORTB &= ~0x01;}

#define NUM_HISTORY                 12
volatile int capVolt;
volatile int capV[NUM_HISTORY];
volatile unsigned char capVindex;

#define PWM_MODE_SET                pwmMode = SLEEP_MODE_IDLE
#define PWM_MODE_CLEAR              pwmMode = SLEEP_MODE_PWR_DOWN
#define IS_PWM_MODE                 (pwmMode == SLEEP_MODE_IDLE)
#define IS_NOT_PWM_MODE             (pwmMode == SLEEP_MODE_PWR_DOWN)
#define PWM_COUNTER_MAX             100
volatile char pwmMode;
volatile char pwmCycleCount;
volatile char pwmCounter;
volatile char pwmValue;

void adcEnable()
{
    ADCSRA = _BV(ADEN) | _BV(ADPS2);
    ADMUX = _BV(REFS0);
}

inline void adcDisable()
{
    ADMUX = 0;
    ADCSRA = 0;
}

void wdtEnable()
{
    MCUSR &= ~_BV(WDRF);
    WDTCR |= _BV(WDCE) | _BV(WDE);
    WDTCR = _BV(WDTIE) | _BV(WDP2);     // 250mS;
}

void saveCapVolt()
{
    capV[capVindex++] = capVolt;
    if(capVindex >= NUM_HISTORY) capVindex = 0;
}

inline unsigned char inCharge()
{
    unsigned char incCount = 0;
    volatile unsigned char actualIndex = capVindex;
    for(unsigned char index = 0; index < NUM_HISTORY; index++)
    {
        if(capV[actualIndex] < (capVolt)) incCount++;
        actualIndex++;
        if(actualIndex >= NUM_HISTORY) actualIndex = 0;
    }
    return (incCount > ((NUM_HISTORY * 3) / 4));
}

void setPwm(int val)
{
    if(val == 0) {
        pwmValue = 0;
        PWM_MODE_CLEAR;
    } else if(val < PWM_COUNTER_MAX) {
        pwmValue = val;
        PWM_MODE_SET;
    } else {
        pwmValue = PWM_COUNTER_MAX;
        PWM_MODE_CLEAR;
    }
}

inline void bypassCurrent()
{
    if(capVolt >= CONV_LIMIT(LIMIT_WARNING_VOLT)) {
        CHARGE_END_ON;
        BYPASS_ON;
    } else if(capVolt < CONV_LIMIT(LIMIT_LOW_VOLT)) {
        CHARGE_END_OFF;
        BYPASS_OFF;
    } else {
        if(capVolt >= CONV_LIMIT(LIMIT_FULL_VOLT)) {
            CHARGE_END_ON;
        } else {
            CHARGE_END_OFF;
        }
        if(inCharge()) {
            BYPASS_PWM((capVolt - CONV_LIMIT(LIMIT_LOW_VOLT)) / 4);
        } else {
            BYPASS_OFF;
        }
    }
}

ISR(TIM0_COMPA_vect)
{
    if(pwmValue == 0) {
        PORTB &= ~0x02;
    } else if(pwmValue == PWM_COUNTER_MAX) {
        PORTB |= 0x02;
    } else {
        pwmCounter++;
        if(pwmCounter > pwmValue) {
            PORTB &= ~0x02;
        } else {
            PORTB |= 0x02;
        }
    }
    if(pwmCounter > PWM_COUNTER_MAX) {
        if(pwmValue == 0) {
            PORTB &= ~0x02;
        } else {
            PORTB |= 0x02;
        }
        pwmCycleCount++;
        pwmCounter = 0;
    }
}

ISR(WDT_vect)
{
}

int main()
{
    unsigned char dataCount;
    DDRB |= 0x07;
    DIDR0 = 0x3C;
//    capVolt = 0;
    dataCount = 0;
    for(unsigned char index = 0; index < NUM_HISTORY; index++)
    {
        capV[index] = 19999;
    }
//    pwmCycleCount = 0;
    PWM_MODE_CLEAR;
    OCR0A = 119;     // 0.1mS
    TCCR0A = _BV(WGM01);
    TCCR0B = _BV(CS00);
    wdtEnable();

    while(1)
    {
        set_sleep_mode(pwmMode);
        sleep_enable();
        sei();
        sleep_cpu();
        sleep_disable();

        if(IS_NOT_PWM_MODE || (pwmCycleCount >= 25)) {
            pwmCycleCount = 0;
            TIMSK0 &= ~_BV(OCIE0A);
            adcEnable();                        // ADCN
            _delay_us(200);
            ADMUX = _BV(REFS0) | 0x02;
            ADCSRA |= _BV(ADSC);
            loop_until_bit_is_set(ADCSRA, ADIF);    // ŏ̕ϊf[^j
            ADCSRA |= _BV(ADIF);

            ADMUX = _BV(REFS0) | 0x03;          // LpV^d擾
            capVolt = 0;
            for(char count = 0; count < 8; count++)
            {
                ADCSRA |= _BV(ADSC);
                loop_until_bit_is_set(ADCSRA, ADIF);
                ADCSRA |= _BV(ADIF);
                capVolt += ADC;                      // ADCϊf[^擾
            }
            capVolt = capVolt >> 3;
            adcDisable();                       // ȓd͂̂ADC~
//            capVolt = CONV_CAP_VOLT(capVolt);
            bypassCurrent();                    // dɂ[ddoCpX
            if(IS_PWM_MODE) {
                TIMSK0 |= _BV(OCIE0A);
            }
            if(dataCount++ > 4) {
                dataCount = 0;
                saveCapVolt();
            }
        }

    }
}
